/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TEXTCELL_H
#define TEXTCELL_H

#include <ktextedit.h>

class ViewGrid;
class QDomDocument;
class QDomElement;

/**
 * The actually displayed text cells
*/
class TextCell : public KTextEdit
{
Q_OBJECT
public:
   enum CellType { NONE, NORMAL, FIXED, GENERATED };

   /**
    * Constructor
    * @param parent The parent widget
    * @param position The cells position in the full grid
    */
    TextCell( QWidget *parent = 0, int position = 0 );

    /// The value of the cell
    int value() const;
    /// Sets the value and type of the cell
    void setValue( int value, CellType type = NONE );

    /// Set possible values
    void setPossible( const QString &poss );

    /// The type of the cell
    CellType type() const { return m_type; }
    /// Sets the type of the cell
    void setType( CellType type );

    /// The cells box
    int box() const { return m_box; }

    /// The cells column
    int col() const { return m_col; }

    /// The cells row
    int row() const { return m_row; };

    ///
    virtual int widthForHeight( int w ) const { return w; };

    /**
     * Saves a cell
     * @param doc The document where saving to
     * @return The Element created
     */
    QDomElement saveCell( QDomDocument &doc );

    /**
     * Loads a cell
     * @param e The element to load this cell from
     * @return True if successful
     */
    bool loadCell( QDomElement &e );

    /**
     * Draws the cell
     * @param p The painter to use to draw the cell
     * @param x The x offset to draw the cell at
     * @param y The y offset to draw the cell at
     * @param size The size to draw the cell
     */
    void draw(QPainter &p, const QRect &cell, const QRect &txtBox, const QFont &small, const QFont &large );

protected slots:
   /// Called when the text changes
   void slotTextChanged( );

protected:
   ///
   virtual void resizeEvent( QResizeEvent* );
   ///
   void keyPressEvent( QKeyEvent *e );

private:
   bool testText( const QString &txt );

   ViewGrid *m_grid;
   CellType m_type;
   int m_position;
   int m_box;
   int m_col;
   int m_row;
};

#endif
