/***************************************************************************
 *   Copyright (C) 2005 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef VIEWGRID_H
#define VIEWGRID_H

#include <qwidget.h>

// forward declarations
class QFont;
class TextCell;
class GridState;
class QPainter;

/**
 * This widget is the actual viewable grid and holds all the text cells.
 */
class ViewGrid : public QWidget
{
Q_OBJECT
public:
   /// The direction for focus setting
   enum FocusDirection { UP, DOWN, LEFT, RIGHT, ONE, BACK_ONE };
   /// How to treat the setting gridset
   enum GridType { NEW, SOLUTION, HINT };

   /// Constructor
   ViewGrid(QWidget *parent = 0, const char *name = 0);

   /// Retrieves a gridstate that matches this view
   GridState grid();

   /// Sets the grid to match the gridstate
   void setGrid( GridState grid, GridType type );

   /// Shows the possible values in the blank cells
   void showPossibles();

   /// Clears the current grid
   void clear();

   /**
    * Saves the current grid state
    * @param fileName The filename to save the grid too
    * @return True if the save was successful
    */
   bool saveGrid( const QString &fileName );

   /**
    * Loads a grid state from the specified name
    * @param fileName The filename of the grid to load
    * @return True if successful
    */
   bool loadGrid( const QString &fileName );

   /// The current large font
   QFont largeText() { return m_largeText; }
   /// The current small font
   QFont smallText() { return m_smallText; }

   /// Moves the focus in the direction specified
   void setCellFocus( int currentCell, FocusDirection dir );

   /// draws the grid on the given QPainter
   void draw( QPainter &p );

signals:
   /// emitted when the grid is complete
   void gridComplete();

protected:
   ///
   virtual void resizeEvent( QResizeEvent* );

protected slots:
   /// called when the actual text contents changes in cells
   void slotCellChanged();

private:
   TextCell *m_pCells[81];
   QFont m_largeText;
   QFont m_smallText;
};

#endif
